package feature.produce;

import feature.produce.model.DTSConfig;
import feature.produce.service.CMDService;
import feature.produce.service.DTSConfigService;
import feature.produce.service.ODPSService;
import feature.produce.service.RedisService;
import feature.produce.util.JSONUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.collections4.MapUtils;
import org.apache.commons.lang3.tuple.Pair;
import org.apache.spark.SparkConf;
import org.apache.spark.api.java.JavaRDD;
import org.apache.spark.api.java.JavaSparkContext;

import java.util.List;
import java.util.Map;

/**
 * @author dyp
 */
@Slf4j
public class ODPSToRedis {
    private static ODPSService odpsService = new ODPSService();
    private static RedisService redisService = new RedisService();
    private static CMDService cmdService = new CMDService();
    private static DTSConfigService dtsConfigService = new DTSConfigService();

    public static void main(String[] args) {

        Map<String, String> argMap = cmdService.parse(args);
        if (MapUtils.isEmpty(argMap)) {
            log.error("args is empty");
            return;
        }
        // ODPS
        DTSConfig config = dtsConfigService.getDTSConfig(argMap);
        if (config == null || !config.selfCheck()) {
            log.error("dts config error");
            return;
        }


        List<Map<String, String>> fieldValues = odpsService.read(config, argMap);
        if (CollectionUtils.isEmpty(fieldValues)) {
            return;
        }

        // RDD
        SparkConf sparkConf = new SparkConf()
                .setAppName("odps sync to redis");
        // .setMaster("local");
        JavaSparkContext jsc = new JavaSparkContext(sparkConf);
        JavaRDD<Map<String, String>> readData = jsc.parallelize(fieldValues);
        JavaRDD<Pair<String, String>> json = readData.map(
                f -> Pair.of(redisService.redisKey(f, config), JSONUtils.toJson(f))
        );

        int partitionNum = fieldValues.size() / 1000 + 1;
        json.repartition(partitionNum).foreachPartition(iterator -> {
            redisService.mSet(iterator, config);
        });
    }

}
