package feature.produce.service;

import com.aliyun.core.utils.StringUtils;
import com.ctrip.framework.apollo.ConfigService;
import com.google.common.reflect.TypeToken;
import feature.produce.model.DTSConfig;
import feature.produce.util.JSONUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Optional;

/**
 * https://help.aliyun.com/zh/maxcompute/user-guide/java-sdk-1/?spm=a2c4g.11174283.0.0.6d0111c1E15lI3
 *
 * @author dyp
 */
@Slf4j
public class DTSConfigService {

    public void init(String env) {
        System.setProperty("app.id", "recommend-feature");

        if (StringUtils.equals(env, "prod")) {
            System.setProperty("apollo.meta", "http://apolloconfig-internal.piaoquantv.com");
        } else {
            System.setProperty("apollo.meta", "http://devapolloconfig-internal.piaoquantv.com");
        }
    }

    public DTSConfig getDTSConfig(Map<String, String> argMap) {

        init(argMap.get("env"));

        List<DTSConfig> dtsConfigs = JSONUtils.fromJson(
                ConfigService.getAppConfig().getProperty("dts.config.v2", ""),
                new TypeToken<List<DTSConfig>>() {
                },
                Collections.emptyList());
        if (CollectionUtils.isEmpty(dtsConfigs)) {
            log.error("DTSConfig not config");
            return null;
        }
        Optional<DTSConfig> optional = dtsConfigs.stream()
                .filter(c -> c.getOdps() != null && StringUtils.equals(c.getOdps().getTable(), argMap.get("table")))
                .findFirst();
        if (!optional.isPresent()) {
            log.error("table {} not config", argMap.get("table"));
            return null;
        }
        return optional.get();
    }
}